@@ -42,7 +42,7 @@ rc.cmpd.get.pubchem <- function(
   get.synonyms = TRUE,
   find.short.lipid.name = TRUE,
   find.short.synonym = TRUE,
-  max.name.length = 20,
+  max.name.length = 30,
   assign.short.name = TRUE,
   get.bioassays = FALSE,
   write.csv = TRUE
@@ -67,7 +67,7 @@ rc.cmpd.get.pubchem <- function(
       warning("pubchem rest triggered a warning.", '\n')
     },
     finally={
-      closeAllConnections()
+      cons <- suppressWarnings(showConnections(all = TRUE)); rm(cons)
     }
   )
   rm(out)
@@ -160,7 +160,7 @@ rc.cmpd.get.pubchem <- function(
           return(NA)
         },
         finally={
-          closeAllConnections()
+          cons <- suppressWarnings(showConnections(all = TRUE)); rm(cons)
         }
       )
       if(is.na(out[1])) next
@@ -200,7 +200,7 @@ rc.cmpd.get.pubchem <- function(
           return(NA)
         },
         finally={
-          closeAllConnections()
+          cons <- suppressWarnings(showConnections(all = TRUE)); rm(cons)
         }
       )
       if(is.na(out[1])) next
@@ -250,7 +250,7 @@ rc.cmpd.get.pubchem <- function(
     parent.cid <- cmpd.cid
     # do.ind <- which(!is.na(cmpd.cid) & !is.na(cmpd.names))
     do.ind <- which(!is.na(cmpd.cid))
-    for(i in 1:length(do.ind)) {
+    for(i in do.ind) {
       Sys.sleep(0.25)
       html <- paste0("https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/cid/",
                      cmpd.cid[i],
@@ -266,12 +266,13 @@ rc.cmpd.get.pubchem <- function(
           return(NA)
         },
         finally={
-          closeAllConnections()
+          cons <- suppressWarnings(showConnections(all = TRUE)); rm(cons)
         }
       )
       if(is.na(out[1])) next
       out <- sort(as.numeric(out))
-      parent.cid[do.ind[i]] <- out[1]
+      parent.cid[i] <- out[1]
+      rm(out)
     }
     cid <- parent.cid
     d <- data.frame(d, "parent.cid" = parent.cid, stringsAsFactors = FALSE)
@@ -377,6 +378,7 @@ rc.cmpd.get.pubchem <- function(
     
     properties <- d[,0]
     for(i in 1:length(cid.l)) {
+      Sys.sleep(0.5)
       keep <- which(!cid.l[[i]]=="NA")
       if(length(keep) == 0) next
       html <- paste0("https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/cid/",
@@ -417,7 +419,7 @@ rc.cmpd.get.pubchem <- function(
           return(NA)
         },
         finally={
-          closeAllConnections()
+          cons <- suppressWarnings(showConnections(all = TRUE)); rm(cons)
         }
       )
       if(is.na(out)) next
@@ -480,6 +482,23 @@ rc.cmpd.get.pubchem <- function(
       for(j in 1:length(out$InformationList$Information)) {
         on <- which(cid ==  cid.l[[i]][keep[j]])
         if(length(on)==0) next
+        if(length(on) > 1) {
+          if(!(any(ls() == "multi.cid"))) {
+            multi.cid <- rep(1, 1)
+            names(multi.cid)[1] <- cid.l[[i]][keep[j]]
+            on <- on[1]
+          } else {
+            if(any(names(multi.cid) == cid.l[[i]][keep[j]])) {
+              which.multi <- which(names(multi.cid) == cid.l[[i]][keep[j]])
+              multi.cid[which.multi] <- multi.cid[which.multi] + 1
+              on <- on[multi.cid[which.multi]]
+            } else {
+              multi.cid <- c(multi.cid, 1)
+              names(multi.cid)[length(multi.cid)] <- cid.l[[i]][keep[j]]
+              on <- on[1]
+            }
+          }
+        }
         tmp <- out$InformationList$Information[[j]]
         if(is.null(tmp$Synonym)) next
         syns <- unlist(tmp$Synonym)
@@ -555,7 +574,7 @@ rc.cmpd.get.pubchem <- function(
           return( data.frame("cid" = rep(NA, 0)))
         },
         finally={
-          closeAllConnections()
+          cons <- suppressWarnings(showConnections(all = TRUE)); rm(cons)
         }
       )
       if(nrow(bd)==0) next
@@ -577,6 +596,12 @@ rc.cmpd.get.pubchem <- function(
     }
   }
   
+  for(i in 1:length(pubchem)) {
+    if(!is.data.frame(pubchem[[i]])) next
+    if(nrow(pubchem[[i]]) == length(ramclustObj$cmpd)) {
+      pubchem[[i]] <- data.frame("cmpd" = ramclustObj$cmpd, pubchem[[i]])
+    }
+  }
   
   if(write.csv) {
     if(is.null(search.name)) {search.name = "pubchem.data"}
